/**
 * A Postgres query which may be fed directly into the `pg` module for
 * execution.
 */
export interface PGQuery {
    /**
     * The SQL query text with placeholders for values. The placeholders refer to
     * a value in the `values` array.
     */
    text: string;
    /**
     * The values used to fill the placeholders in `text`.
     */
    values: Array<any>;
}
/**
 * The representation of a SQL query. Call `compile` to turn it into a SQL
 * string with value placeholders.
 *
 * This object is immutable. Instead of changing the object, new `SQLQuery`
 * values will be returned.
 *
 * The constructor for this class is private and may not be called.
 */
export declare class SQLQuery implements PGQuery {
    /**
     * A template string tag that interpolates literal SQL with placeholder SQL
     * values.
     */
    static query(strings: TemplateStringsArray, ...values: Array<any>): SQLQuery;
    /**
     * Joins multiple queries together and puts a seperator in between if a
     * seperator was defined.
     */
    static join(queries: Array<SQLQuery>, seperator?: string): SQLQuery;
    /**
     * Creates a new query with the raw text.
     */
    static raw(text: string): SQLQuery;
    /**
     * Creates a new query with the value. This value will be turned into a
     * placeholder when the query gets compiled.
     */
    static value(value: any): SQLQuery;
    /**
     * Creates an identifier query. Each name will be escaped, and the
     * names will be concatenated with a period (`.`).
     */
    static identifier(...names: Array<any>): SQLQuery;
    /**
     * The internal array of SQL items. This array is never mutated, only cloned.
     */
    private readonly _items;
    /**
     * Storage for our memoized compiled query.
     */
    private _query;
    private constructor(items);
    /**
     * The SQL query text with placeholders for values. The placeholders refer to
     * a value in the `values` array.
     */
    readonly text: string;
    /**
     * The values used to fill the placeholders in `text`.
     */
    readonly values: Array<any>;
    /**
     * Compiles this SQL query into a Postgres query. Memoized so it only does the
     * work once.
     */
    compile(): PGQuery;
}
/**
 * The interface we actually expect people to use.
 */
export interface SQL {
    (strings: TemplateStringsArray, ...values: Array<any>): SQLQuery;
    join(queries: Array<SQLQuery>, seperator?: string): SQLQuery;
    raw(text: string): SQLQuery;
    value(value: any): SQLQuery;
    ident(...names: Array<any>): SQLQuery;
}
export declare const sql: SQL;
